/***********************************************************************************************
   	           wmsensormon - a sensor monitor for WindowMaker thanks to wmbp6 by RedSeb
VERSION	     : 1.2.1
DATE   	     : 12/17/2004
AUTHOR       : Liverbugg <liverbugg@rinux.org>
REQUIREMENTS : Lm_sensors, i2c, WindowMaker, Motherboard with supported sensors
USE          : To monitor the sensors on your motherboard
************************************************************************************************
   			    This Program is licensed under the GPL
***********************************************************************************************/



#include 	<sys/param.h>
#include 	<sys/types.h>
#include	"standards.h"
#include 	<X11/Xlib.h>
#include 	<X11/xpm.h>
#include 	<X11/extensions/shape.h>
#include 	<sensors/sensors.h>
#include 	<sensors/chips.h>
#include 	<sensors/error.h>
#include 	<string.h>
#include 	"../wmgeneral/wmgeneral.h"
#include 	"../wmgeneral/misc.h"
#include 	"wmsensormon_master2.xpm"
#include 	"wmsensormon_mask.xbm"

#define		ERR_MODULES	"\tCan't find \"/etc/sensors.conf\".\n" \
   				"\tCheck the instalation of lm_sensors.\n"
#define		ERR_CONF_SENS   "\tProblems with \"/etc/sensors.conf\".\n"
#define		ERR_SENS_DETECT	"\tNo suitable sensors detected.\n"
#define		DELAI_DCLICK	10000
#define		CPT_DCLICK	10
#define		VERSION		"1.2.1"
#define 	DATE		"12/17/2004"

#define         VOLT1 1
#define         VOLT1_MIN 11
#define         VOLT1_MAX 21
#define         FAN1 31
#define         FAN2 32
#define         FAN3 33
#define         FAN4 34
#define         FAN1_MIN 41
#define         FAN2_MIN 43
#define         FAN3_MIN 45
#define         FAN4_MIN 47
#define         TEMP1 51
#define         TEMP1_OVER 53
#define         TEMP2 54
#define         TEMP2_OVER 56
#define         TEMP3 57
#define         TEMP3_OVER 59

void	usage(void);
void 	version(void);
void    display_units(int smp, int unit, int offset);
void 	display_values(int, int, int);
void    DrawBar(int sx, int sy, int w, int h, float percent, int dx, int dy);

int main(int argc, char **argv)
{
   long 	delay = 100000;
   FILE		*config_file;
   FILE         *logfile = 0;
   char         *logfilename;
   double	fan1_d, fan2_d, fan3_d, fan4_d,
                temp1_d, temp2_d, temp3_d,
                t_max_d, fan_min_d, v_max_d, v_min_d, volt0_d,
                multi = 10, syst_max_d;
   int		fan1 = 0, fan2 = 0,
                temp1 = 0, temp2 = 0, temp3 = 0,
                fan1_p = 0, fan2_p = 0,
                temp1_p = 0, temp2_p = 0, temp3_p = 0,
                volt0_p = 0, t_max = 0, fan_min = 0, volt0 = 0, v_min = 0, v_max = 0, syst_max = 0,
                add = 0, unit = 0, smp = 0, swap = 0, swapfan = 0, log = 0,
                t_warn = 0, syst_warn = 0, fan_warn = 0;
   const	sensors_chip_name *chipset;  
   const        sensors_feature_data *feature;
   int		chip_nr = 0, opt = 0,
                offset = 0, offset2 = 1, i = 0, d = 0, logd = 30000;
   float        volt0_per;
   XEvent	Event;
   char		loop,
		time_out,
                reading = 1;
   
   if((config_file = fopen("/etc/sensors.conf", "r")) == NULL)
   {
     puts(ERR_MODULES);
     exit(ERROR);
   }
   else
     {
       if(sensors_init(config_file))
	 {
	   puts(ERR_CONF_SENS);
	   exit(ERROR);
	 }
     }   
   fclose(config_file);

   while(argc > i)
     {
       if(!strcmp(argv[i], "-s1"))
	 {
	   smp = 1;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-s2"))
	 {
	   smp = 2;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-s"))
	 {
	   smp = 1;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-sw"))
	 {
	   swap = 1;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-swf"))
	 {
	   swapfan = 1;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-l"))
	 {
	   log = 1;
	   logfilename = "wmsensormonlog";
	   if (!(logfile = fopen(logfilename, "w"))) {
	     log = 0;
	     fprintf(stderr,"Can't write to log file.\n");
	   }
	   fprintf(logfile, "temp1     temp2     temp3     volt0     fan1      fan2\n");
	   fflush(logfile);
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {	
	       printf("\nUsing default log time of %d\n",logd);
	     }
	   else
	     {
	       logd = atoi(argv[i + 1]);
	       i ++;
	     }
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-r"))
	 {
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {	    
	       printf("\nInvalid options in -r\n");
	       exit(ERROR);
	     }
	   delay = atol(argv[i + 1]);
	   i ++;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-dt"))
	 {
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {	    
	       printf("\nInvalid options in -dt\n");
	       exit(ERROR);
	     }
	   t_max = atoi(argv[i + 1]) * 10;
	   i ++;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-ds"))
	 {
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {	    
	       printf("\nInvalid options in -ds\n");
	       exit(ERROR);
	     }
	   syst_max = atoi(argv[i + 1]) * 10;
	   i ++;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-df"))
	 {
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {	    
	       printf("\nInvalid options in -df\n");
	       exit(ERROR);
	     }
	   fan_min = atoi(argv[i + 1]);
	   i ++;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-wt"))
	 {
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {	    
	       printf("\nInvalid options in -wt\n");
	       exit(ERROR);
	     }
	   t_warn = atoi(argv[i + 1]) * 10;
	   i ++;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-ws"))
	 {
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {	    
	       printf("\nInvalid options in -ws\n");
	       exit(ERROR);
	     }
	   syst_warn = atoi(argv[i + 1]) * 10;
	   i ++;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-wf"))
	 {
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {	    
	       printf("\nInvalid options in -wf\n");
	       exit(ERROR);
	     }
	   fan_warn = atoi(argv[i + 1]);
	   i ++;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-lv"))
	 {
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {	    
	       printf("\nInvalid options in -lv\n");
	       exit(ERROR);
	     }	 
	   else if(!strcmp(argv[i + 1], "%"))
	     {
	       if((i + 2 >= argc)||(argv[i + 2][0] == '-')||(i + 3 >= argc)||(argv[i + 3][0] == '-'))
		 {	    
		   printf("\nInvalid options in -lv\n");
		   exit(ERROR);
		 }
	       v_max = atoi(argv[i + 3]) + ((atoi(argv[i + 2]) * atoi(argv[i + 3])) / 100);
	       if((i + 2 >= argc)||(argv[i + 2][0] == '-')||(i + 3 >= argc)||(argv[i + 3][0] == '-'))
		 {	    
		   printf("\nInvalid options in -lv\n");
		   exit(ERROR);
		 }
	       v_min = atoi(argv[i + 3]) - ((atoi(argv[i + 2]) * atoi(argv[i + 3])) / 100);
	       opt = 1;
	     }
	   else
	     {
	       if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
		 {	    
		   printf("\nInvalid options in -lv\n");
		   exit(ERROR);
		 }	     
	       v_max = atoi(argv[i + 1]);
	       i ++;
	       
	     if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	       {	    
		 printf("\nInvalid options in -lv\n");
		 exit(ERROR);
	       }	     
	     v_min = atoi(argv[i + 1]);
	     i ++;
	     
	     opt = 1;
	   }
	 }
       else if(!strcmp(argv[i], "-f"))
	 {
	   unit = 1;
	   opt = 1;
	 }
       else if(!strcmp(argv[i], "-c"))
	 {
	   if((i + 1 >= argc)||(argv[i + 1][0] == '-'))
	     {       
	       printf("\nInvalid options in -c\n");
	       exit(ERROR);
	     }
	   
	   chip_nr = atol(argv[i + 1]);
	   i ++;
	   opt = 1;
	 }
       
       else if((!strcmp(argv[i], "-h"))||(!strcmp(argv[i], "--help")))
	 {
	   usage();
	 }
       else if((!strcmp(argv[i], "-v"))||(!strcmp(argv[i], "--version")))
	 {
	   version();
	 }
       i ++;
     }
   
   if(opt == 0)
     {
       printf("\nInvalid option or no options given.\n");
       usage();
     }

   if (!chip_nr)
     {
       do
	 {
	   feature = NULL;
	   chipset = sensors_get_detected_chips(&chip_nr);
	   
	   /* Is there a chip here? */
	   if (!chipset)
	     chip_nr = 0;
	   else
	     {
	       int nr1 = 0, nr2 = 0;
	       
	       /* Does this chip have a temperature sensor? */
	       do
		 {
		   feature = sensors_get_all_features(*chipset, &nr1, &nr2);
		   if ((feature) && (!strcmp(feature->name, "temp1")))
		     break;
		 } while (feature);
	       
	       /* Is this chip suitable? */
	       if (feature)
		 break;
	     }

	 } while (chip_nr > 0);
     }
   else
     {
       chip_nr--;
       feature = NULL;
       chipset = sensors_get_detected_chips(&chip_nr);
       printf("chipnum: %i\n", chip_nr);
       
       /* Is there a chip here? */
       if (!chipset)
	 chip_nr = 0;
       else
	 {
	   int nr1 = 0, nr2 = 0;
	   
	   /* Does this chip have a temperature sensor? */
	   do
	     {
	       feature = sensors_get_all_features(*chipset, &nr1, &nr2);
	       if ((feature) && (!strcmp(feature->name, "temp1")))
		 break;
	     } while (feature);
	   
	   /* Is this chip suitable? */
	   if (!feature)
	     chip_nr = 0;
	 }
     }

   /* Did we find a suitable chip? */
   if (!chip_nr)
     {
       puts(ERR_SENS_DETECT);
       exit(ERROR);
     }
   
   /*if (!chipset)
     {
       printf("Could not find a sensor-chip :/\n");
       if (!chip_nr)
	 printf("lm-sensors modules seems not be loaded\n");
       else
	 printf("probably invalid sensor-chip selected\n");
       exit(2);
       }*/
   
   if (swap)
     {
       sensors_get_feature(*chipset,TEMP2_OVER, &t_max_d);
       sensors_get_feature(*chipset,TEMP1_OVER, &syst_max_d);
     }
   else
     {
       if (smp)
	 {
	   sensors_get_feature(*chipset,TEMP1_OVER, &t_max_d);
	   sensors_get_feature(*chipset,TEMP3_OVER, &syst_max_d);
	 }
       else
	 {
	   sensors_get_feature(*chipset,TEMP1_OVER, &t_max_d);
	   sensors_get_feature(*chipset,TEMP2_OVER, &syst_max_d);
	 }
     }

   sensors_get_feature(*chipset,FAN1_MIN, &fan_min_d);
   sensors_get_feature(*chipset,VOLT1_MIN, &v_min_d);
   sensors_get_feature(*chipset,VOLT1_MAX, &v_max_d);

   if (!t_max)
     t_max = (int)(t_max_d * 10);
   if (!syst_max)
     syst_max = (int)(syst_max_d * 10);
   if (!fan_min)
     fan_min = (int)(fan_min_d / 1);
   if (!v_min)
     v_min = ((int)((v_min_d)*100));
   if (!v_max)
     v_max = ((int)((v_max_d)*100));

   t_warn = t_max;
   syst_warn = syst_max;
   fan_warn = fan_min;

   if(unit == 1)
     {
       multi = 1.8;
       add = 32;
       t_max = ((t_max / 10) * 1.8) + 32;
       syst_max = ((syst_max / 10) * 1.8) + 32;
       t_warn = ((t_warn / 10) * 1.8) + 32;
       syst_warn = ((syst_warn / 10) * 1.8) + 32;
     }
   
   if(v_max >= 1000)
     v_max = v_max / 10;
   if(v_min >= 1000)
     v_min = v_min / 10;
   if(v_max < 100)
     v_max = v_max * 10;
   if(v_min < 100)
     v_min = v_min * 10;
   
   openXwindow(argc,argv,wmsensormon_master2_xpm,wmsensormon_mask_bits,wmsensormon_mask_width,wmsensormon_mask_height);
   
   delay = delay / 2;

   if(log)
     fprintf(logfile, "Using sensor number %i\n", chip_nr);

   while(TRUE)
     {
       if(log == 1)
	 {
	   if(d >= logd)
	     {
	       if(unit == 1)
		 {
		   if (temp1 > t_max || temp2 > t_max || temp3 > syst_max || fan1 < fan_min || fan2 < fan_min || volt0 < v_min || volt0 > v_max)
		     fprintf(logfile, "%2.0f       %2.0f        %2.0f        %2.2f      %2.0f      %2.0f WW\n", (float)temp1, (float)temp2, (float)temp3, (float)volt0/100, (float)fan1, (float)fan2);
		   else
		     fprintf(logfile, "%2.0f       %2.0f        %2.0f        %2.2f      %2.0f      %2.0f\n", (float)temp1, (float)temp2, (float)temp3, (float)volt0/100, (float)fan1, (float)fan2);
		 }
	       else
		 {
		   if (temp1 > t_max || temp2 > t_max || temp3 > syst_max || fan1 < fan_min || fan2 < fan_min || volt0 < v_min || volt0 > v_max)
		     fprintf(logfile, "%2.1f      %2.1f      %2.1f      %2.2f      %2.0f      %2.0f WW\n", (float)temp1/10, (float)temp2/10, (float)temp3/10, (float)volt0/100, (float)fan1, (float)fan2);
		   else
		     fprintf(logfile, "%2.1f      %2.1f      %2.1f      %2.2f      %2.0f      %2.0f\n", (float)temp1/10, (float)temp2/10, (float)temp3/10, (float)volt0/100, (float)fan1, (float)fan2);
		 }
	       fflush(logfile);
	       d=0;
	     }
	   else
	     {
	       d++;
	     }
	 }
       if(reading)
	 {
	   sensors_get_feature(*chipset,TEMP1, &temp1_d);
	   sensors_get_feature(*chipset,TEMP2, &temp2_d);
	   sensors_get_feature(*chipset,TEMP3, &temp3_d);
	   sensors_get_feature(*chipset,VOLT1, &volt0_d);
	   sensors_get_feature(*chipset,FAN1, &fan1_d);
	   sensors_get_feature(*chipset,FAN2, &fan2_d);
	   sensors_get_feature(*chipset,FAN3, &fan3_d);
	   sensors_get_feature(*chipset,FAN4, &fan4_d);
	 
	   if(swap == 1)
	     {
	       if(smp >= 1)
		 {
		   temp1 = (int)(temp2_d * multi + add);
		   temp2 = (int)(temp3_d * multi + add);
		   temp3 = (int)(temp1_d * multi + add);
		 }
	       else
		 {
		   temp1 = (int)(temp2_d * multi + add);
		   temp2 = (int)(temp1_d * multi + add);
		   temp3 = (int)(temp3_d * multi + add);
		 }
	     }
	   else
	     {
	       temp1 = (int)(temp1_d * multi + add);
	       temp2 = (int)(temp2_d * multi + add);
	       temp3 = (int)(temp3_d * multi + add);
	     }
	   if(swapfan == 1)
	     {
	       fan1 = (int)(fan3_d * 1);
	       fan2 = (int)(fan4_d * 1);
	     }
	   else
	     {
	       fan1 = (int)(fan1_d * 1);
	       fan2 = (int)(fan2_d * 1);
	     }

	   volt0 = volt0_d * 100;
	   
	   if(smp == 1)
	     {
	       if(temp1 != temp1_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(temp1 > t_max)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(temp1 > t_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(0, 87 + offset, 23, 7, 5, 4);
		   copyXPMArea(69, 87 + offset, 5, 7, 23, 4);
		   display_values(temp1, 0, offset2);
		   display_units(unit, offset, 0);
		   temp1_p = temp1;
		 }
	     
	       if(temp2 != temp2_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(temp2 > t_max)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(temp2 > t_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(0, 87 + offset, 23, 7, 5, 18);
		   copyXPMArea(75, 87 + offset, 5, 7, 23, 18);
		   display_values(temp2, 14, offset2);
		   display_units(unit, offset, 14);
		   temp2_p = temp2;
		 }
	       
	       if(volt0 != volt0_p)
		 {
		   volt0_per = ((float)(volt0-v_min)/(float)(v_max-v_min)) * 100;
		   DrawBar(0, 109, 58, 4, volt0_per, 3, 58);
		   volt0_p = volt0;
		 }
	       if(temp3 != temp3_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(temp3 > syst_max)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(temp3 > syst_warn)
		     {
		       offset = 14;
		       offset2 = 14 ;
		     }
		   copyXPMArea(23, 87 + offset, 23, 7, 5, 32);
		   display_values(temp3, 28, offset2);
		   display_units(unit, offset, 28);
		   temp3_p = temp3;
		 }
	       if(fan1 != fan1_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(fan1 < fan_min)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(fan1 < fan_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(46, 87 + offset, 23, 7, 5, 46);
		   display_values(fan1, 42, offset2);
		   fan1_p = fan1;
		 }
	     }
	   
	   else if(smp == 2)
	     {
	       if(temp1 != temp1_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(temp1 > t_max)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(temp1 > t_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(0, 87 + offset, 23, 7, 5, 4);
		   copyXPMArea(69, 87 + offset, 5, 7, 23, 4);
		   display_values(temp1, 0, offset2);
		   display_units(unit, offset, 0);
		   temp1_p = temp1;
		 }
	       
	       if(temp2 != temp2_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(temp2 > syst_max)
		     {
		       offset = 7;
		       offset2 = 7 ;
		     }
		   else if(temp2 > syst_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(0, 87 + offset, 23, 7, 5, 18);
		   copyXPMArea(75, 87 + offset, 5, 7, 23, 18);
		   display_values(temp2, 14, offset2);
		   display_units(unit, offset, 14);
		   temp2_p = temp2;
		 }
	       
	       if(volt0 != volt0_p)
		 {
		   volt0_per = ((float)(volt0-v_min)/(float)(v_max-v_min)) * 100;
		   DrawBar(0, 109, 58, 4, volt0_per, 3, 58);
		   volt0_p = volt0;
		 }
	       if(fan1 != fan1_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(fan1 < fan_min)
		     {
		       offset = 7;
		       offset2 = 7 ;
		     }
		   else if(fan1 < fan_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(46, 87 + offset, 23, 7, 5, 32);
		   copyXPMArea(69, 87 + offset, 5, 7, 23, 32);
		   display_values(fan1, 28, offset2);
		   fan1_p = fan1;
		 }
	       if(fan2 != fan2_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(fan2 < fan_min)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(fan2 < fan_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(46, 87 + offset, 23, 7, 5, 46);
		   copyXPMArea(75, 87 + offset, 5, 7, 23, 46);
		   display_values(fan2, 42, offset2);
		   fan2_p = fan2;
		 }
	     }
	   else
	     {
	       if(temp1 != temp1_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(temp1 > t_max)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(temp1 > t_warn)
		     {
		       offset = 14;
		       offset2 = 14 ;
		     }
		   copyXPMArea(0, 87 + offset, 23, 7, 5, 4);
		   display_values(temp1, 0, offset2);
		   display_units(unit, offset, 0);
		   temp1_p = temp1;
		 }
	       
	       if(temp2 != temp2_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(temp2 > syst_max)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(temp2 > syst_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(23, 87 + offset, 23, 7, 5, 18);
		   display_values(temp2, 14, offset2);
		   display_units(unit, offset, 14);
		   temp2_p = temp2;
		 }
	       
	       if(volt0 != volt0_p)
		 {
		   volt0_per = ((float)(volt0-v_min)/(float)(v_max-v_min)) * 100;
		   DrawBar(0, 109, 58, 5, volt0_per, 3, 57);
		   volt0_p = volt0;
		 }
	       if(fan1 != fan1_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(fan1 < fan_min)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(fan1 < fan_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(46, 87 + offset, 23, 7, 5, 32);
		   copyXPMArea(69, 87 + offset, 5, 7, 23, 32);
		   display_values(fan1, 28, offset2);
		   fan1_p = fan1;
		 }
	       if(fan2 != fan2_p)
		 {
		   offset = 0;
		   offset2 = 0;
		   if(fan2 < fan_min)
		     {
		       offset = 7;
		       offset2 = 7;
		     }
		   else if(fan2 < fan_warn)
		     {
		       offset = 14;
		       offset2 = 14;
		     }
		   copyXPMArea(46, 87 + offset, 23, 7, 5, 46);
		   copyXPMArea(75, 87 + offset, 5, 7, 23, 46);
		   display_values(fan2, 42, offset2);
		   fan2_p = fan2;
		 }
	     }
	 }
       reading = 1 - reading;
       RedrawWindow();
       if(XCheckMaskEvent(display, ButtonPressMask, &Event))
	 {
	   if(Event.type == ButtonPress)
	     {
	       loop = TRUE;
	       time_out = 0;
	       while((time_out < CPT_DCLICK) && loop)
		 {
		   if(XCheckMaskEvent(display, ButtonPressMask, &Event))
		     {
		       if(Event.type == ButtonPress)
			 {
			   loop = FALSE;
			   if(unit == 0)
			     {  
			       multi = 1.8;
			       add = 32;
			       unit = 1;
			       t_max = ((t_max / 10) * 1.8) + 32 + 1;
			       syst_max = ((syst_max / 10) * 1.8) + 32 + 1;
			       t_warn = ((t_warn / 10) * 1.8) + 32 + 1;
			       syst_warn = ((syst_warn / 10) * 1.8) + 32 + 1;
			     }
			   else
			     {
			       multi = 10;
			       add = 0;
			       unit = 0;
			       t_max = ((t_max - 32) / 1.8) * 10 + 6;
			       syst_max = ((syst_max - 32) / 1.8) * 10 + 6;
			       t_warn = ((t_warn - 32) / 1.8) * 10 + 6;
			       syst_warn = ((syst_warn - 32) / 1.8) * 10 + 6;
			     }
			 }
		     }    
		   time_out ++;
		   usleep(DELAI_DCLICK);
		 }
	     }
	 }
       usleep(delay);      
     }
}  

void display_bar(int sx, int sy, int w, int h, float percent, int dx, int dy)
{
  static int tx;
  int temp = sx ;
  tx = (float)((float)w * ((float)percent / (float)100.0));
  
  while(temp <= tx)
    {
      copyXPMArea(temp, sy, 1, h, dx, dy);
      temp++;
    }
  
  copyXPMArea(sx, sy, tx, h, dx, dy);
  
  copyXPMArea(sx, sy + h + 3, w - tx, h, dx + tx, dy);
}

void DrawBar(int sx, int sy, int w, int h, float percent, int dx, int dy)
{
  static int tx;
  tx = (float)((float)w * ((float)percent / (float)100.0));
  
  copyXPMArea(sx, sy, tx, h, dx, dy);
  
  copyXPMArea(sx, sy + h + 3, w - tx, h, dx + tx, dy);
}


void display_units(int unit, int offset, int offset2)
{
  if(unit == 0)
    {
      copyXPMArea(51, 70 + offset, 2, 2, 45, 10 + offset2);
      copyXPMArea(66, 65 + offset, 6, 7, 53, 4 + offset2);
    }
  else
    {
      copyXPMArea(54, 70 + offset, 2, 2, 45, 10 + offset2);
      copyXPMArea(73, 65 + offset, 6, 7, 53, 4 + offset2);
    }
}

void display_values(int temp, int offset, int offset2)
{
  char text[5],
    num1,
    num2,
    num3,
    num4;
  
  sprintf(text, "%03d", temp);
  num1 = (text[0] - '0');
  num2 = (text[1] - '0');
  num3 = (text[2] - '0');
  num4 = (text[3] - '0');
  if(num1)
    copyXPMArea(5 * num1, 65 + offset2, 5, 7, 33, 4 + offset);
  else
    copyXPMArea(60, 65 + offset2, 5, 7, 33, 4 + offset);
  copyXPMArea(5 * num2, 65 + offset2, 5, 7, 39 + 1, 4 + offset);
  copyXPMArea(5 * num3, 65 + offset2, 5, 7, 45 + 2, 4 + offset);
  copyXPMArea(5 * num4, 65 + offset2, 5, 7, 51 + 3, 4 + offset);
}


void usage(void)
{
  puts("\nwmsensormon "VERSION" - "DATE" - monitoring applet for motherboard sensors.\nIncludes Fahrenheit and Celsius.\nIf text is in red, set the -d* value for that field\nIf text is in orange, set the -w* value for that field\nExclude decimal points from all numbers\nLogging currently logs to the file wmsensormonlog in the directory it was run from\n\n"
       "Options : -h, --help     this help screen\n"
       "          -v, --version  print version information\n"
       "          -c             sensor chip number to use (default = 0)\n"
       "          -f             show temperatures in Fahrenheit\n"
       "          -s             default SMP mode (mode #1)\n"
       "          -s1            SMP mode #1 (CPU 1, CPU 2, SYS, fan)\n"
       "          -s2            SMP mode #2 (CPU 1, CPU 2, fan 1, fan 2)\n"
       "          -sw            switch CPU and SYS temperature readings\n"
       "                         use with -s1 or -s2 if SYS temp is first\n"
       "          -swf           switch fans to read fan3 and fan4\n"
       "          -r             refresh time (in microseconds, default 100000)\n"
       "          -l             enable logging with default time, (default 30000)\n" 
       "          -ln            enable logging with specified time, (1000 = 1 min)\n" 
       "          -dt            danger CPU temperature, default: system value (i.e. 60)\n"
       "                         input values in Celsius\n"
       "          -ds            danger SYS temperature, default: system value (i.e. 60)\n"
       "                         input values in Celsius\n"
       "          -df            danger fan speed, default: system value (i.e. 3000)\n"
       "          -wt            warning CPU temperature, default: danger value\n"
       "                         input values in Celsius\n"
       "          -ws            warning SYS temperature, default: danger value\n"
       "                         input values in Celsius\n"
       "          -wf            warning fan speed, default: danger value\n"
       "          -lv            limit CPU core volts, default is system value\n"
       "                 either: max voltage first, min voltage second (i.e. 180 170)\n"
       "                 or:     percent difference first, value second (i.e. % 5 175)\n\n"
       "Example command lines:\n\n"   
       "danger temp, fan, volt:  wmsensormon -dt 50 -ds 35 -df 5000 -lv 183 167 -f\n\n"
       "using percent and SMP :  wmsensormon -dt 50 -ds 35 -df 5000 -lv % 5 175 -s -f\n\n"
       "warn and danger temps :  wmsensormon -dt 50 -ds 35 -wt 45 -ws 30\n\n"
       "You must specify at least one command line argument, do to the fact that not everyone has the same CPU, voltage, or fan setup.\n\n"
       "<liverbugg@rinux.org> http://wmsensormon.sourceforge.net\n"
       );
  exit(OK);
}

void version(void)
{
  puts("\nwmsensormon "VERSION" - "DATE"\n"
       "<liverbugg@rinux.org> http://wmsensormon.sourceforge.net\n"
       );
  exit(OK);
}  
